// -*- C++ -*-
/*!
 * @file CORBA_Util.h
 * @brief CORBA utility template classes
 * @date $Date: 2007-04-26 15:24:41 $
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2006
 *     Task-intelligence Research Group,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id: CORBA_SeqEx.h 706 2008-04-16 00:36:15Z n-ando $
 *
 */

#ifndef CORBA_Util_h
#define CORBA_Util_h

#include <rtm/RTC.h>
#include <ace/Guard_T.h>
#include <ace/Thread_Mutex.h>
#include <ace/Recursive_Thread_Mutex.h>


typedef ACE_Guard<ACE_Thread_Mutex> Guard;
typedef ACE_Read_Guard<ACE_Thread_Mutex> Read_Guard;
typedef ACE_Write_Guard<ACE_Thread_Mutex> Write_Guard;

typedef ACE_Guard<ACE_Recursive_Thread_Mutex> Guard_r;
typedef ACE_Read_Guard<ACE_Recursive_Thread_Mutex> Read_Guard_r;
typedef ACE_Write_Guard<ACE_Recursive_Thread_Mutex> Write_Guard_r;


/*!
 * CORBA sequence wp[ev[g֐
 *
 * for_each ()
 * find ()
 * push_back ()
 * insert ()
 * front()
 * back ()
 * erase ()
 * clear ()
 *
 */
namespace CORBA_Sequence_Util
{
  /*!
   * @if jp
   * @brief CORBA sequence ɑ΂ functor Kp
   *
   * CORBA sequence SĂ̗vfɑ΂āA^ꂽ functor KpB
   * functor  void functor(CORBA sequence ̗vf) ̌`ƂKvB
   *
   * @param seq Functor Kp CORBA sequence
   * @param f CORBA sequence ̗vf Functor
   *
   * @return SĂ̗vf Functor
   *
   * @else
   *
   * @brief Apply the functor to all CORBA sequence elements
   *
   * Apply the given functor to the given CORBA sequence.
   * functor should be void functor(CORBA sequence element).
   *
   * @param seq CORBA sequence to be applied the functor
   * @param f A functor to process CORBA sequence elements
   *
   * @return Functor that processed all CORBA sequence elements
   *
   * @endif
   */
  template <class CorbaSequence, class Functor>
  Functor for_each(const CorbaSequence& seq, Functor f)
  {
    CORBA::ULong len;
    len = seq.length();
    for (CORBA::ULong i = 0; i < len; ++i)
      {
	f(seq[i]);
      }
    return f;
  }


  /*!
   * @if jp
   * @brief CORBA sequence ̒ functor ɓKvf̃CfbNXԂ
   *
   * CORBA sequence SĂ̗vfɑ΂āA^ꂽ functor KpA
   * functor  true Ԃvf̃CfbNXԂB
   * functor  bool functor(const CORBA sequence ̗vf) ̌`ƂA
   * Kvfɑ΂ true ԂKvB
   *
   * @param seq Functor Kp CORBA sequence
   * @param f CORBA sequence vf Functor
   *
   * @return FunctorɓKvf̃CfbNXB
   *         ȂƂ -1 ԂB
   *
   * @else
   *
   * @brief Return the index of CORBA sequence element that functor matches 
   *
   * This operation applies the given functor to the given CORBA sequence,
   * and returns the index of the sequence element that the functor matches.
   * The functor should be bool functor(const CORBA sequence element) type,
   * and it would return true, if the element matched the functor.
   *
   * @param seq CORBA sequence to be applied the functor
   * @param f A functor to process CORBA sequence elements
   *
   * @return The index of the element that functor matches.
   *         If no element found, it would return -1.
   *
   * @endif
   */
  template <class CorbaSequence, class Functor>
  CORBA::Long find(const CorbaSequence& seq, Functor f)
  {
    CORBA::ULong len(seq.length());
    for (CORBA::ULong i = 0; i < len; ++i)
      {
	if (f(seq[i])) return (CORBA::Long)i;
      }
    return -1;
  }

  /*!
   * @if jp
   * @brief CORBA sequence ̍Ōɗvfǉ
   *
   * CORBA sequence ̍Ōɗ^ꂽvfǉB
   * CORBA sequence ͎̒IɊgB
   *
   * @param seq vfǉ CORBA sequence
   * @param elem ǉvf
   *
   * @else
   *
   * @brief Push the new element back to the CORBA sequence
   *
   * Add the given element to the last of CORBA sequence.
   * The length of the CORBA sequence will be expanded automatically.
   *
   * @param seq CORBA sequence to be added a new element
   * @param elem The new element to be added to the CORBA sequence
   *
   * @endif
   */
  template <class CorbaSequence, class SequenceElement>
  void push_back(CorbaSequecne& seq, SequenceElement elem)
  {
    CORBA::ULong len(seq.length());
    seq.length(len + 1);
    seq[len] = item;
  }


  /*!
   * @if jp
   * @brief CORBA sequence ɗvf}
   *
   * CORBA sequence  index ̈ʒuɗvfB
   * index  ^ꂽ@CORBA sequence ̍ő index 傫ꍇ
   * Ō̗vfƂĉB
   * CORBA sequence ͎̒IɊgB
   *
   * @param seq vfǉ CORBA sequence
   * @param elem ǉvf
   * @param index vfǉʒu
   *
   * @else
   *
   * @brief Insert the element to the CORBA sequence
   *
   * Insert a new element in the given position to the CORBA sequence.
   * If the given index is greater than the length of the sequence,
   * the given element is pushed back to the last of the sequence.
   * The length of the CORBA sequence will be expanded automatically.
   *
   * @param seq The CORBA sequence to be inserted a new element
   * @param elem The new element to be inserted the sequence
   * @param index The inserting position
   *
   * @endif
   */
  template <class CorbaSequence, class SequenceElement>
  void insert(CorbaSequence& seq, SequenceElement& elem, CORBA::ULong index)
  {
    CORBA::ULong len(seq.length());
    if (index > len) push_back(seq, elem);

    seq.length(len + 1);
    for (CORBA::ULong i = len; i >= index; --i)
      {
	seq[i + 1] = seq[i];
      }
    seq[index] = elem;
  }


  /*!
   * @if jp
   * @brief CORBA sequence ̐擪vf擾
   *
   * CORBA sequence ̐擪vf擾Bseq[0] ƓB
   *
   * @param seq vf擾 CORBA sequence
   *
   * @return 擾vf
   *
   * @else
   *
   * @brief Get the front element of the CORBA sequence
   *
   * This operation returns seq[0].
   *
   * @param seq The CORBA sequence to be get the element
   *
   * @endif
   */
  template <class CorbaSequence, class SequenceElement>
  SequenceElement& front(CorbaSequence& seq)
  {
    return seq[0];
  }


  /*!
   * @if jp
   * @brief CORBA sequence ̖vf擾
   *
   * CORBA sequence ̖vf擾Bseq[seq.length() - 1] ƓB
   *
   * @param seq vf擾 CORBA sequence
   *
   * @return 擾vf
   *
   * @else
   *
   * @brief Get the last element of the CORBA sequence
   *
   * This operation returns seq[seq.length() - 1].
   *
   * @param seq The CORBA sequence to be get the element
   *
   * @endif
   */	
  template <class CorbaSequence, class SequenceElement>
  SequenceElement& back(CorbaSequence& seq)
  {
    return seq[seq.length() - 1];
  }


  /*!
   * @if jp
   * @brief CORBA sequence ̎w肳ꂽʒu̗vf폜
   *
   * w肳ꂽCfbNX̗vf폜B
   * 폜ꂽvf͋l߂Asequence ̒1B
   *
   * @param seq vf폜 CORBA sequence
   * @param index 폜vf̃CfbNX
   *
   * @else
   *
   * @brief Erase the element of the specified index
   *
   * This operation removes the element of the given index.
   * The other elements are closed up around the hole.
   *
   * @param seq The CORBA sequence to be get the element
   * @param index The index of the element to be removed
   *
   * @endif
   */
  template <class CorbaSequence>
  void erase(CorbaSequence& seq, CORBA::ULong index)
  {
    CORBA::ULong len(seq.length());
    if (index > len) return;

    for (CORBA::ULong i = index; i < len - 1; ++i)
      {
	seq[i] = seq[i + 1];
      }
    seq.length(len - 1);
  }

  /*!
   * @if jp
   * @brief CORBA sequence ̑Svf폜
   *   
   * CORBA sequence ̑Svf폜B
   * seq.length(0) ƓB
   *
   * @else
   *
   * @brief Erase all elements of the CORBA sequence
   *
   * same as seq.length(0).
   *
   * @endif
   */
  template <class CorbaSequence>
  void clear(CorbaSequence& seq)
  {
    seq.length(0);
  }
  // End of CORBA sequence helper functions



  //============================================================
  // CORBA sequence extention class
  //============================================================

  /*!
   * @if jp
   * @brief  bNtf[^
   * @else
   * @brief  Data with mutex lock
   * @endif
   */
  template <class T>
  struct LockedStruct
  {
    ACE_Thread_Mutex lock;
    T                data;
  };
  
  /*!
   * @if jp
   * @class SequenceEx
   * @brief CORBA::sequence gNX
   *
   * ̃NX CORBA  sequence ^g std::vector ̃C^[tF[X
   * 񋟂 (Ⴆ size(), max_size(), empty(), resize(), insert(),
   * erase(), erase_if(), push_back(), pop_back(), find()).
   * CORBA  sequence ^pĂ邽߁ACORBA  sequence ^
   * Iy[V(like operator=(), maximum(), length(), operator[])
   * gp\łB
   * 
   * @since 0.4.0
   *
   * @else
   *
   * @brief CORBA::sequence extention class
   *
   * This class extends CORBA sequence type, and provides std::vector like
   * interfaces (like size(), max_size(), empty(), resize(), insert(),
   * erase(), erase_if(), push_back(), pop_back(), find()).
   * Since this class inherits CORBA sequence class, user can also use CORBA
   * sequence interfaces (like operator=(), maximum(), length(), operator[]).
   *
   * @since 0.4.0
   *
   * @endif
   */ 
  template <class CorbaSequence, class SequenceItem, class Mutex>
  class SequenceEx
    : public CorbaSequence
  {
  public:
    /*!
     * @if jp
     *
     * @brief ftHgRXgN^
     *
     * ftHgRXgN^BftHgRXgN^ł́A
     * NXCORBA::sequence ̒ 0 ɃZbgB
     * 
     * @else
     *
     * @brief Default constructor
     *
     * Default constructor. This default constructor sets the sequence length
     * of CorbaSequence base class.
     *
     * @endif
     */
    SequenceEx(): CorbaSequence(0) {};
    
    
    /*!
     * @if jp
     *
     * @brief CorbaSequence ̃Rs[RXgN^
     *
     * CorbaSequence^̃Rs[RXgN^B
     * ^ꂽ CorbaSequence ̓eRs[B
     *
     * @param _sq CorbaSequence `̃Rs[
     *
     * @else
     *
     * @brief Copy constructor from CorbaSequence
     *
     * This constructor copies sequence contents from given CorbaSequence
     * to this object.
     *
     * @param _sq Copy source of CorbaSequence type
     *
     * @endif
     */
    SequenceEx(const CorbaSequence& _sq)
      : CorbaSequence(0)
    {
      this->length(_sq.length());
      CORBA::ULong len(this->length());
      for (CORBA::ULong i = 0; i < len; ++i)
	(*this)[i] = _sq[i];
    };
    
    
    /*!
     * @if jp
     *
     * @brief Rs[RXgN^
     *
     * @param _sq Rs[
     *
     * @else
     *
     * @brief Copy constructor
     *
     * @param _sq Copy source.
     *
     * @endif
     */
    SequenceEx(const SequenceEx& _sq)
      : CorbaSequence(0)
    {
      this->length(_sq.length());
      CORBA::ULong len(this->length());
      for (CORBA::ULong i = 0; i < len; ++i)
	(*this)[i] = _sq[i];
    };
    
    
    /*!
     * @if jp
     *
     * @brief Zq
     *
     * @param _sq 
     *
     * @return 
     *
     * @else
     *
     * @brief Assignment operator
     *
     * @param _sq Copy source.
     *
     * @return An assignment result
     *
     * @endif
     */
    SequenceEx& operator=(const SequenceEx& _sq)
    {
      this->length(_sq.length());
      CORBA::ULong len(this->length());
      for (CORBA::ULong i = 0; i < len; ++i)
	(*this)[i] = _sq[i];
      return *this;
    };
    
    
    /*!
     * @if jp
     *
     * @brief CorbaSequence ̑Zq
     *
     * @param _sq 
     *
     * @return 
     *
     * @else
     *
     * @brief Assignment operator from CorbaSequence
     *
     * @param _sq Copy source.
     *
     * @return An assignment result
     *
     * @endif
     */
    SequenceEx& operator=(const CorbaSequence& _sq)
    {
      this->length(_sq.length());
      CORBA::ULong len(this->length());
      for (CORBA::ULong i = 0; i < len; ++i)
	(*this)[i] = _sq[i];
      return *this;
    };
    
    
    /*!
     * @if jp
     *
     * @brief fXgN^
     *
     * @else
     *
     * @brief Destructor
     *
     * @endif
     */
    virtual ~SequenceEx()
    {
      this->length(0);
    };
    
    
    /*!
     * @if jp
     *
     * @brief TCY擾
     *
     * ̃Iy[V̓V[PX̃TCYԂB
     * CorbaSequence::length() ƓB
     *
     * @return V[PX̃TCY
     *
     * @else
     *
     * @brief Get size of this sequence
     *
     * This operation returns the size of the sequence.
     * This is same as CorbaSequence::length().
     *
     * @return The size of the sequence.
     *
     * @endif
     */
    inline CORBA::ULong size() { return this->length(); }
    
    
    /*!
     * @if jp
     *
     * @brief i[\ȍő̃TCY擾
     *
     * ̃Iy[V̓V[PX݂̌̊i[\ȍő̃TCYԂB
     * CorbaSequence::maximum() ƓB
     *
     * @return V[PXɊi[\ȍő̃TCY
     *
     * @else
     *
     * @brief Get current maximum size of this sequence
     *
     * This operation returns the current maximum size of the sequence.
     * This is same as CorbaSequence::maximum().
     *
     * @return The maximum size of the sequence.
     *
     * @endif
     */
    inline CORBA::ULong max_size() { return this->maximum(); }
    
    
    /*!
     * @if jp
     *
     * @brief V[PX󂩂ǂׂ
     *
     * ̃Iy[V̓V[PX󂩂ǂ bool lŕԂB
     * TCY 0 Ȃ trueAłȂ false ԂB
     *
     * @return V[PX󂩂ǂ bool l
     *
     * @else
     *
     * @brief Test whether the sequence is empty
     *
     * This operation returns bool value whether the sequence is empty.
     * If the size of the sequence is 0, this operation returns true,
     * and in other case this operation returns false.
     *
     * @return The bool value whether the sequence is empty.
     *
     * @endif
     */
    inline bool empty() { return (this->length() == 0) ? true : false; }
    
    
    /*!
     * @if jp
     *
     * @brief V[PXTCY
     *
     * ̃Iy[V̓V[PX̒ύXB
     * ݂̒傫ȃTCY^ꂽꍇA item ŁA
     * VɃAP[gꂽ߂B
     * ݂̒菬TCY^ꂽꍇACorabSequence Ɠl
     * ]ȃV[PX̗vf͍폜B
     *
     * @param new_size VV[PX̃TCY
     * @param item@Ȃ̃V[PX𖄂߂vf
     *
     * @else
     *
     * @brief Resize the length of the sequence
     *
     * This operation resizes the length of the sequence.
     * If longer length than current sequence length is given,
     * newly allocated rooms will be assigned by element given by the argument.
     * If shorter length than current sequence length is given,
     * the excessive element of a sequence is deleted like behavior of
     * CorabSequence
     *
     * @param new_size The new size of the sequence
     * @param item     Sequence element to be assigned to new rooms.
     *
     * @endif
     */
    void resize(CORBA::ULong new_size, SequenceItem& item)
    {
      ACE_Write_Guard<Mutex> gaurd(lock);
      CORBA::ULong len(this->length);
      if (new_size > len) // Widen sequence
	{
	  this->length(new_size);
	  for (CORBA::ULong i = len; i < new_size; ++i)
	    (*this)[i] = item;
	}
      else if (new_size < len) // shorten sequence
	{
	  this->length(new_size);
	}
    }
    
    
    /*!
     * @if jp
     *
     * @brief V[PXɗvf}
     *
     * ̃Iy[V̓V[PX̓rɗvf}B
     * sequence ͎̒IɊgB
     *
     * @param position Vvf}ꏊ
     * @param item }V[PX̗vf
     *
     * @else
     *
     * @brief Insert a new item to the sequence
     *
     * This operation inserts a new item to the sequence.
     * The length of the sequence will be expanded automatically.
     *
     * @param position The position of new inserted item.
     * @param item     Sequence element to be inserted.
     *
     * @endif
     */
    void insert(CORBA::ULong position, const SequenceItem& item)
    {
      ACE_Write_Guard<Mutex> gaurd(lock);
      CORBA::ULong len(this->length());
      // allowed position: [0, len]
      if (position > len) throw; // InvalidParameter("Invalid index");
      
      // widen sequence
      this->length(len + 1);
      // shift items
      for (CORBA::ULong i = (len - 1); i > position; --i)
	{
	  (*this)[i] = (*this)[i-1];
	}
      (*this)[position] = item;
    }
    
    
    /*!
     * @if jp
     *
     * @brief V[PX̗vf폜
     *
     * ̃Iy[V̓V[PX̗vf폜
     * sequence ͎̒IɏkB
     *
     * @param position 폜V[PXvf̏ꏊ
     *
     * @return 폜ꂽvf
     *
     * @else
     *
     * @brief Erase an item of the sequence
     *
     * This operation erases an item from the sequence.
     *
     * @param position The position of erased item.
     *
     * @return The element which was removed
     *
     * @endif
     */
    SequenceItem erase(CORBA::ULong position)
    {
      ACE_Write_Guard<Mutex> gaurd(lock);
      CORBA::ULong len(this->length());
      // allowed position: [0, len)
      if (position > (len - 1)) throw; // InvalidParameter("Invalid index");
      
      // keep erased item
      SequenceItem erased((*this)[position]);
      // shift remaining items
      for (CORBA::ULong i = position; i < (len - 1); ++i)
	{
	  (*this)[i] = (*this)[i+1];
	}
      // shorten sequence
      this->length(len - 1);
      return erased;
    }
    
    /*!
     * @if jp
     *
     * @brief V[PX̗vfqɂč폜
     *
     * ̃Iy[V͏qƂė^ꂽ֐IuWFNg
     * ^̂ƂÃV[PX̗vf폜B
     *
     * @param f 폜V[PX肷p
     *
     * @return 폜ꂽvf
     *
     * @else
     *
     * @brief Erase an item according to the given predicate
     *
     * This operation erases an item according to the given predicate.
     *
     * @param f The predicate functor to decide deletion.
     *
     * @return The element which was removed
     *
     * @endif
     */
    template <class Predicate>
    SequenceItem erase_if(Predicate f)
    {
      ACE_Write_Guard<Mutex> gaurd(lock);
      CORBA::ULong len(this->length());
      for (CORBA::ULong i = 0; i < len; ++i)
	if (f((*this)[i]))
	  return erase(i);
      throw;
    }
    
    
    /*!
     * @if jp
     *
     * @brief vfŌɒǉ
     *
     * ̃Iy[V͗^ꂽvfV[PX̍ŌɒǉB
     * sequence ͎̒IɊgB
     *
     * @param item ǉ邷IuWFNg
     *
     * @else
     *
     * @brief Append an item to the end of the sequence.
     *
     * This operation push back an item to the end of the sequence.
     * The length of the sequence will be expanded automatically.
     *
     * @param item The object to be added to the end of the sequnce.
     *
     * @endif
     */ 
    void push_back(const SequenceItem& item)
    {
      ACE_Write_Guard<Mutex> gaurd(lock);
      CORBA::ULong len(this->length());
      this->length(len + 1);
      (*this)[len] = item;
    }

    /*!
     * @if jp
     *
     * @brief Ō̗vf폜
     *
     * ̃Iy[V̓V[PX̍Ō̗vf폜B
     * sequence ͎̒IɏkB
     *
     * @else
     *
     * @brief Remove an item from the end of the sequence.
     *
     * This operation removes an item from the end of the sequence.
     * The length of the sequence will be diminished automatically.
     *
     * @endif
     */
    void pop_back()
    {
      ACE_Write_Guard<Mutex> gaurd(lock);
      CORBA::ULong len(this->length());
      this->len(len - 1);
    }

    /*!
     * @if jp
     * @brief sequencepfunctor
     * @else
     * @brief Functor for sequence searches
     * @endif
     */
    template <class F>
    SequenceItem find(F f) const
    {
      ACE_Read_Guard<Mutex> guard(lock);
      CORBA::ULong len(this->length());
      for (CORBA::ULong i = 0; i < len; ++i)
	if (f((*this)[i]))
	  return (*this)[i];
      throw; // InvalidParameter("Not found.");
    }

    /*!
     * @if jp
     * @brief f[^bNpmutex
     * @else
     * @brief mutex for data locks
     * @endif
     */
    mutable Mutex lock;
  }; // class SequenceEx
  
  
}; // namespace CORBA_Sequence_Util

#endif // CORBA_Util_h
